@@ -23,7 +23,8 @@ BROWSER_NAMES = {'ff': "_make_ff",
                  'htmlunitwithjs' : "_make_htmlunitwithjs",
                  'android': "_make_android",
                  'iphone': "_make_iphone",
-                 'safari': "_make_safari"
+                 'safari': "_make_safari",
+                 'edge': "_make_edge"
                 }
 
 class _BrowserManagementKeywords(KeywordGroup):
@@ -85,6 +86,7 @@ class _BrowserManagementKeywords(KeywordGroup):
         | android          | Android       |
         | iphone           | Iphone        |
         | safari           | Safari        |
+        | edge             | Edge          |
 
 
         Note, that you will encounter strange behavior, if you open
@@ -94,7 +96,7 @@ class _BrowserManagementKeywords(KeywordGroup):
         http://selenium-grid.seleniumhq.org/faq.html#i_get_some_strange_errors_when_i_run_multiple_internet_explorer_instances_on_the_same_machine
 
         Optional 'remote_url' is the url for a remote selenium server for example
-        http://127.0.0.1/wd/hub.  If you specify a value for remote you can
+        http://127.0.0.1:4444/wd/hub. If you specify a value for remote you can
         also specify 'desired_capabilities' which is a string in the form
         key1:val1,key2:val2 that will be used to specify desired_capabilities
         to the remote server. This is useful for doing things like specify a
@@ -157,7 +159,7 @@ class _BrowserManagementKeywords(KeywordGroup):
         | Create Webdriver            | PhantomJS    | service_args=${service args}              |                         |
 
         Example for Robot Framework < 2.8:
-        | # debug IE driver |                   |                  |                                | 
+        | # debug IE driver |                   |                  |                                |
         | ${kwargs}=        | Create Dictionary | log_level=DEBUG  | log_file=%{HOMEPATH}${/}ie.log |
         | Create Webdriver  | Ie                | kwargs=${kwargs} |                                |
         """
@@ -580,6 +582,13 @@ class _BrowserManagementKeywords(KeywordGroup):
         return self._generic_make_browser(webdriver.Safari,
                 webdriver.DesiredCapabilities.SAFARI, remote, desired_capabilities)
 
+    def _make_edge(self , remote , desired_capabilities , profile_dir):
+        if hasattr(webdriver, 'Edge'):
+            return self._generic_make_browser(webdriver.Edge,
+                webdriver.DesiredCapabilities.EDGE, remote, desired_capabilities)
+        else:
+            raise ValueError("Edge is not a supported browser with your version of Selenium python library. Please, upgrade to minimum required version 2.47.0.")
+
     def _generic_make_browser(self, webdriver_type , desired_cap_type, remote_url, desired_caps):
         '''most of the make browser functions just call this function which creates the
         appropriate web-driver'''
