@@ -1,14 +1,40 @@
 from selenium.webdriver.common.keys import Keys
 from selenium.webdriver.common.action_chains import ActionChains
+from selenium.webdriver.remote.webelement import WebElement
 from Selenium2Library import utils
 from Selenium2Library.locators import ElementFinder
+from Selenium2Library.locators import CustomLocator
 from keywordgroup import KeywordGroup
 
+try:
+    basestring  # attempt to evaluate basestring
+    def isstr(s):
+        return isinstance(s, basestring)
+except NameError:
+    def isstr(s):
+        return isinstance(s, str)
+
 class _ElementKeywords(KeywordGroup):
 
     def __init__(self):
         self._element_finder = ElementFinder()
 
+    # Public, get element(s)
+
+    def get_webelement(self, locator):
+        """Returns the first WebElement matching the given locator.
+
+        See `introduction` for details about locating elements.
+        """
+        return self._element_find(locator, True, True)
+
+    def get_webelements(self, locator):
+        """Returns list of WebElement objects matching locator.
+
+        See `introduction` for details about locating elements.
+        """
+        return self._element_find(locator, False, True)
+
     # Public, element lookups
 
     def current_frame_contains(self, text, loglevel='INFO'):
@@ -54,6 +80,23 @@ class _ElementKeywords(KeywordGroup):
                           "its text was '%s'." % (locator, expected, actual)
             raise AssertionError(message)
 
+    def element_should_not_contain(self, locator, expected, message=''):
+        """Verifies element identified by `locator` does not contain text `expected`.
+
+        `message` can be used to override the default error message.
+
+        Key attributes for arbitrary elements are `id` and `name`. See
+        `Element Should Contain` for more details.
+        """
+        self._info("Verifying element '%s' does not contain text '%s'."
+                   % (locator, expected))
+        actual = self._get_text(locator)
+        if expected in actual:
+            if not message:
+                message = "Element '%s' should not contain text '%s' but " \
+                          "it did." % (locator, expected)
+            raise AssertionError(message)
+
     def frame_should_contain(self, locator, text, loglevel='INFO'):
         """Verifies frame identified by `locator` contains `text`.
 
@@ -73,7 +116,9 @@ class _ElementKeywords(KeywordGroup):
 
         If this keyword fails, it automatically logs the page source
         using the log level specified with the optional `loglevel` argument.
-        Giving `NONE` as level disables logging.
+        Valid log levels are DEBUG, INFO (default), WARN, and NONE. If the
+        log level is NONE or below the current active log level the source
+        will not be logged.
         """
         if not self._page_contains(text):
             self.log_source(loglevel)
@@ -93,6 +138,24 @@ class _ElementKeywords(KeywordGroup):
         """
         self._page_should_contain_element(locator, None, message, loglevel)
 
+    def locator_should_match_x_times(self, locator, expected_locator_count, message='', loglevel='INFO'):
+        """Verifies that the page contains the given number of elements located by the given `locator`.
+
+        See `introduction` for details about locating elements.
+
+        See `Page Should Contain Element` for explanation about `message` and
+        `loglevel` arguments.
+        """
+        actual_locator_count = len(self._element_find(locator, False, False))
+        if int(actual_locator_count) != int(expected_locator_count):
+            if not message:
+                message = "Locator %s should have matched %s times but matched %s times"\
+                            %(locator, expected_locator_count, actual_locator_count)
+            self.log_source(loglevel)
+            raise AssertionError(message)
+        self._info("Current page contains %s elements matching '%s'."
+                   % (actual_locator_count, locator))
+
     def page_should_not_contain(self, text, loglevel='INFO'):
         """Verifies the current page does not contain `text`.
 
@@ -234,6 +297,16 @@ class _ElementKeywords(KeywordGroup):
             raise AssertionError("Could not determine position for '%s'" % (locator))
         return element.location['x']
 
+    def get_element_size(self, locator):
+        """Returns width and height of element identified by `locator`.
+
+        The element width and height is returned.
+        Fails if a matching element is not found.
+        """
+        element = self._element_find(locator, True, True)
+
+        return element.size['width'], element.size['height']
+
     def get_value(self, locator):
         """Returns the value attribute of element identified by `locator`.
 
@@ -248,6 +321,14 @@ class _ElementKeywords(KeywordGroup):
         """
         return self._get_text(locator)
 
+    def clear_element_text(self, locator):
+        """Clears the text value of text entry element identified by `locator`.
+
+        See `introduction` for details about locating elements.
+        """
+        element = self._element_find(locator, True, True)
+        element.clear()
+
     def get_vertical_position(self, locator):
         """Returns vertical position of element identified by `locator`.
 
@@ -274,7 +355,7 @@ class _ElementKeywords(KeywordGroup):
 
     def click_element_at_coordinates(self, locator, xoffset, yoffset):
         """Click element identified by `locator` at x/y coordinates of the element.
-        Cursor is moved and the center of the element and x/y coordinates are 
+        Cursor is moved and the center of the element and x/y coordinates are
         calculted from that point.
 
         Key attributes for arbitrary elements are `id` and `name`. See
@@ -321,11 +402,11 @@ class _ElementKeywords(KeywordGroup):
     def drag_and_drop_by_offset(self, source, xoffset, yoffset):
         """Drags element identified with `source` which is a locator.
 
-        Element will be moved by xoffset and yoffset.  each of which is a
+        Element will be moved by xoffset and yoffset, each of which is a
         negative or positive number specify the offset.
 
         Examples:
-        | Drag And Drop | myElem | 50 | -35 | # Move myElem 50px right and 35px down. |
+        | Drag And Drop By Offset | myElem | 50 | -35 | # Move myElem 50px right and 35px down. |
         """
         src_elem = self._element_find(source, True, True)
         ActionChains(self._current_browser()).drag_and_drop_by_offset(src_elem, xoffset, yoffset).perform()
@@ -384,7 +465,7 @@ class _ElementKeywords(KeywordGroup):
         element = self._element_find(locator, True, False)
         if element is None:
             raise AssertionError("ERROR: Element %s not found." % (locator))
-        ActionChains(self._current_browser()).click_and_hold(element).release(element).perform()
+        ActionChains(self._current_browser()).release(element).perform()
 
     def open_context_menu(self, locator):
         """Opens context menu on element identified by `locator`."""
@@ -414,18 +495,15 @@ return !element.dispatchEvent(evt);
 
     def press_key(self, locator, key):
         """Simulates user pressing key on element identified by `locator`.
-
-        `key` is either a single character, or a numerical ASCII code of the key
-        lead by '\\'.
-
+        `key` is either a single character, a string, or a numerical ASCII code of the key
+        lead by '\\\\'.
         Examples:
         | Press Key | text_field   | q |
-        | Press Key | login_button | \\13 | # ASCII code for enter key |
+        | Press Key | text_field   | abcde |
+        | Press Key | login_button | \\\\13 | # ASCII code for enter key |
         """
         if key.startswith('\\') and len(key) > 1:
             key = self._map_ascii_key_code_to_key(int(key[1:]))
-        #if len(key) > 1:
-        #    raise ValueError("Key value '%s' is invalid.", key)
         element = self._element_find(locator, True, True)
         #select it
         element.send_keys(key)
@@ -534,6 +612,13 @@ return !element.dispatchEvent(evt);
     def get_matching_xpath_count(self, xpath):
         """Returns number of elements matching `xpath`
 
+        One should not use the xpath= prefix for 'xpath'. XPath is assumed.
+
+        Correct:
+        | count = | Get Matching Xpath Count | //div[@id='sales-pop']
+        Incorrect:
+        | count = | Get Matching Xpath Count | xpath=//div[@id='sales-pop']
+
         If you wish to assert the number of matching elements, use
         `Xpath Should Match X Times`.
         """
@@ -543,6 +628,13 @@ return !element.dispatchEvent(evt);
     def xpath_should_match_x_times(self, xpath, expected_xpath_count, message='', loglevel='INFO'):
         """Verifies that the page contains the given number of elements located by the given `xpath`.
 
+        One should not use the xpath= prefix for 'xpath'. XPath is assumed.
+
+        Correct:
+        | Xpath Should Match X Times | //div[@id='sales-pop'] | 1
+        Incorrect:
+        | Xpath Should Match X Times | xpath=//div[@id='sales-pop'] | 1
+
         See `Page Should Contain Element` for explanation about `message` and
         `loglevel` arguments.
         """
@@ -556,16 +648,53 @@ return !element.dispatchEvent(evt);
         self._info("Current page contains %s elements matching '%s'."
                    % (actual_xpath_count, xpath))
 
+    # Public, custom
+    def add_location_strategy(self, strategy_name, strategy_keyword, persist=False):
+        """Adds a custom location strategy based on a user keyword. Location strategies are
+        automatically removed after leaving the current scope by default. Setting `persist`
+        to any non-empty string will cause the location strategy to stay registered throughout
+        the life of the test.
+
+        Trying to add a custom location strategy with the same name as one that already exists will
+        cause the keyword to fail.
+
+        Custom locator keyword example:
+        | Custom Locator Strategy | [Arguments] | ${browser} | ${criteria} | ${tag} | ${constraints} |
+        |   | ${retVal}= | Execute Javascript | return window.document.getElementById('${criteria}'); |
+        |   | [Return] | ${retVal} |
+
+        Usage example:
+        | Add Location Strategy | custom | Custom Locator Strategy |
+        | Page Should Contain Element | custom=my_id |
+
+        See `Remove Location Strategy` for details about removing a custom location strategy.
+        """
+        strategy = CustomLocator(strategy_name, strategy_keyword)
+        self._element_finder.register(strategy, persist)
+
+    def remove_location_strategy(self, strategy_name):
+        """Removes a previously added custom location strategy.
+        Will fail if a default strategy is specified.
+
+        See `Add Location Strategy` for details about adding a custom location strategy.
+        """
+        self._element_finder.unregister(strategy_name)
+
     # Private
 
     def _element_find(self, locator, first_only, required, tag=None):
         browser = self._current_browser()
-        elements = self._element_finder.find(browser, locator, tag)
-        if required and len(elements) == 0:
-            raise ValueError("Element locator '" + locator + "' did not match any elements.")
-        if first_only:
-            if len(elements) == 0: return None
-            return elements[0]
+        if isstr(locator):
+            elements = self._element_finder.find(browser, locator, tag)
+            if required and len(elements) == 0:
+                raise ValueError("Element locator '" + locator + "' did not match any elements.")
+            if first_only:
+                if len(elements) == 0: return None
+                return elements[0]
+        elif isinstance(locator, WebElement):
+            elements = locator
+        # do some other stuff here like deal with list of webelements
+        # ... or raise locator/element specific error if required
         return elements
 
     def _frame_contains(self, locator, text):
@@ -633,6 +762,14 @@ return !element.dispatchEvent(evt);
             key = chr(key_code)
         return key
 
+    def _map_named_key_code_to_special_key(self, key_name):
+        try:
+           return getattr(Keys, key_name)
+        except AttributeError:
+           message = "Unknown key named '%s'." % (key_name)
+           self._debug(message)
+           raise ValueError(message)
+
     def _parse_attribute_locator(self, attribute_locator):
         parts = attribute_locator.rpartition('@')
         if len(parts[0]) == 0:
@@ -642,7 +779,7 @@ return !element.dispatchEvent(evt);
         return (parts[0], parts[2])
 
     def _is_element_present(self, locator, tag=None):
-        return (self._element_find(locator, True, False, tag=tag) != None)
+        return (self._element_find(locator, True, False, tag=tag) is not None)
 
     def _page_contains(self, text):
         browser = self._current_browser()
@@ -682,4 +819,3 @@ return !element.dispatchEvent(evt);
             raise AssertionError(message)
         self._info("Current page does not contain %s '%s'."
                    % (element_name, locator))
-
